/**********************************************************************
 *
 *  umatrix.h  -  Declarations for Curve fit.
 *  Matrix operations are based on standard algorithms in
 *  Chap 6 of P. M. Embree, B Kimble, "C Language Algorithms for Digital
 *  Signal Processing," Prentice-Hall 1991.
 *
 *  27 Feb 00 Initial write - Bob Larkin
 *  3 April 01 - Documented call
 *
 **********************************************************************/

struct MAT
    {
    int            rows;
    int            cols;
    double**       ptr;
    };

/****************************************************************************
 *
 * fit_curve(...)  -  Function to find the least-squares polynomial curve
 * fit that is put into a column vector cf[ ] that must be o elements or more.
 * Input data y[] a function of x[] and weight w[], all of length n (0, n-1).
 *
 * Returns 1 unless matrix function fails. Then returns 0 with
 * an error summary put into caller supplied global variable mat_err.
 *
 */

extern int  fit_curve(double*,    /* cf vector of returned coefficients */
                      double*,    /* x vector of dependent variable */
                      double*,    /* y vector of measured values */
                      double*,    /* w vector of weights for measured values */
                      int,        /* n, number of measured values */
                      int);       /* o, order of fitted polynomial */

extern unsigned int mat_err;      /* Supplied by calling program */
