/**********************************************************************
 *
 *       util.c  -  General support routines for uhfa.c
 *                        Version 3.31
 *
 * NOTICE: Copyright (c) 2000-2005 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm  and,
 *
 * Transferred form UHFA.C 22 Nov 00 -  RSL
 * Updated to 3.10 14 Jan 2004 RSL
 * 3.20 Added pua43_tick 28 Jul 04 RSL
 * 3.23 Put k_ keyboard definition to .CFG file RSL 22 Sept 04
 * 3.26 Added 'J' cmd to quit() to try to keep "$$$" response. RSL 28 Oct 04.
 * 3.28 Removed tfreqL as a CFG item, saved in vfo[].last_tfreq.
 *      Added VFOs as separate from transverters RSL 30 Nov 04
 * 3.31 Removed state stuff to reduce file size, moved to ustate.c RSL 1 Feb 05
 *
 */

#include <alloc.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <graphics.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <dos.h>
#include <io.h>
#include <direct.h>
#include "util.h"
#include "moonsun.h"
#include "uhfa.h"
#include "lbox.h"
#include "u_code.h"
#include "async.h"
#include "gifsave.h"
#include "umatrix.h"
/* Rev 3.50 include "bmp_defs.h" */
#include "uhf_kbd.h"

 void vert_cursor1(int);
 void vert_cursor2(int, int);
 void delayu(unsigned int);
 void do_gif_file(void);
 void call2dos(void);
 void print_big(char, int, int);
 void print_super_row(char, int, int);
 void print_medium(char, int);
 void print_medium_1(char, int);
 void print_medium_2(char, int);
 void dsp_cmd(int, int, int, int, int);
 void show_last_cmd(char* ct);
 void show_beacon(void);
 void ctrlc_handler(int sig);
 void fpe_handler(int sig);
 char *left_justify(char *str1, char *str2, int len2, char fill);
 void point1(int, int);
 void point5(int, int);
 void tiny_time(int, int, int);
 void tiny_char(int, int, int);
 void micro_char(int, int, int);
 void bar_graph(int, int, unsigned int, int);
 int  exit_modalbox(void);
 void beacon_modalbox(void);
 void scs_modalbox(void);
 unsigned int scs_edit_line(struct lbox_info*, struct edit_info*);
 void beep(int);
 void quit(void);

/* These belong in uhelp.h, that doesn't exist */
 void show_help(void);
 void license_info(void);
 void license(void);
 void warranty(void);

/******************  Functions ***********************/
/* vert_cursor1(int hpos) - This stops all processing, places a vertical
 * cursor down the waterfall and spectral displays, and displays the number of
 * Hz at the bottom. The parameter hpos is an integer (0, 511) indicating the
 * screen position in pixels, within the 512 pixel spectral area. This is
 * converted to frequency using the global variable, display_width.
 * Returns on any key being pressed, except screen save.
 * RSL  22 Dec 02
 */
void vert_cursor1(int hpos)
	{
	int          fr, k;
	unsigned int stat;
	long int     lkey;

	/* Save a vertical line, one pixel wide, at the hpos location.
	 * The scroll areas are available since all operation is stopped.
	 */

	/* Set viewport to entire spectral area */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	/* Get the line data from the top spectral box: */
	for(k=0; k<=128; k++)  *(big_area_ptr1+k) = getpixel(hpos, k);
	/* Get the line data from the bottom spectral box: */
	for(k=147; k<=468; k++)  *(big_area_ptr1+k) = getpixel(hpos, k);
	/* Get a box to place frequency value in */
	setviewport(X1L+hpos-20, 469, X1L+hpos+20, 479, DO_CLIP);
	getimage(0, 0, 40, 10, big_area_ptr1+1000);
	clearviewport();

	/* Put lines and marking into place */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	setcolor(c_plot_bold);
	line(hpos, 1, hpos, 126);
	line(hpos, 147, hpos, 468);
	setcolor(c_background);
	rectangle(hpos-20, 469, hpos+20, 479);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(hpos, 469);
	fr=near_int(df[fft_bw]*((double)hpos)/(double)(display_width+1));
	sprintf(cc, "%d", fr);
	outtext(cc);

	for(;;)			/* Wait for keyboard hit, or screen save  */
	    {
	    if ( kbhit() )
		{
		stat = _bios_keybrd(_KEYBRD_SHIFTSTATUS);
		/* Make left and right shift keys equals (bits 0 and 1) and
		 * mask out Caps Lock, Num Lock & Insert (bits 5, 6, and 7).*/
		if (stat&2)   stat |= 1;
		stat &= 0X1D;
		lkey = ((long)_bios_keybrd(_KEYBRD_READ)) + (((long)stat)<<16);
		if (lkey==GIF_C || lkey==GIF_L)
		    {
		    show_last_cmd("SCNSAV");
		    /* Rev 3.50 Removed BMP option */
		    do_gif_file();
		    }
		else
		    break;		/* Key was hit, not screen save */
		}
	    }

	/* Restore display to original & return from function */
	/* Set viewport to entire spectral area */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	/* Put the line data back to top spectral box: */
	for(k=0; k<=128; k++)  putpixel(hpos, k, *(big_area_ptr1+k));
	/* Put the line data back to bottom spectral box: */
	for(k=147; k<=468; k++)  putpixel(hpos, k, *(big_area_ptr1+k));
	/* Replace the screen for the marker data area */
	setviewport(X1L+hpos-20, 469, X1L+hpos+20, 479, DO_CLIP);
	putimage(0, 0, big_area_ptr1+1000, COPY_PUT);
	}

/* vert_cursor2(hpos1, hpos2) - This stops all processing, places a vertical
 * cursor down the waterfall and spectral displays, and displays the number of
 * Hz at the bottom. The parameter hpos is an integer (0, 511) indicating the
 * screen position in pixels, within the 512 pixel spectral area. This is
 * converted to frequency using the global variable, display_width.
 * Returns on any key being pressed.
 * RSL  22 Dec 02
 */
void vert_cursor2(int hpos1, int hpos2)
	{
	int have;
	int fr1, fr2, k;
	unsigned int stat;
	long int     lkey;

	have=(hpos1+hpos2)/2;
	/* Save vertical lines, one pixel wide, at the hpos location.
	 * The scroll areas are available since all operation is stopped.
	 */

	/* Set viewport to entire spectral area */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	/* Get line data from the top spectral box: */
	for(k=0; k<=128; k++)
	    {
	    *(big_area_ptr1+k) = getpixel(hpos1, k);
	    *(big_area_ptr1+1000+k) = getpixel(hpos2, k);
	    }
	/* Get line data from the bottom spectral box: */
	for(k=147; k<=468; k++)
	    {
	    *(big_area_ptr1+k) = getpixel(hpos1, k);
	    *(big_area_ptr1+1000+k) = getpixel(hpos2, k);
	    }
	/* Get a box to place frequency value in */
	setviewport(X1L+have-50, 469, X1L+have+50, 479, DO_CLIP);
	getimage(0, 0, 100, 10, big_area_ptr1+2000);
	clearviewport();

	/* Put lines and marking into place */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	setcolor(c_plot_bold);
	line(hpos1, 1, hpos1, 126);
	line(hpos1, 147, hpos1, 468);
	line(hpos2, 1, hpos2, 126);
	line(hpos2, 147, hpos2, 468);

	setcolor(c_background);
	rectangle(have-50, 469, have+50, 479);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(have, 469);
	fr1=near_int(df[fft_bw]*((double)hpos1)/(double)(display_width+1));
	fr2=near_int(df[fft_bw]*((double)hpos2)/(double)(display_width+1));
	sprintf(cc, "%d   %d", fr1, fr2);
	outtext(cc);

	for(;;)			/* Wait for keyboard hit, or screen save  */
	    {
	    if ( kbhit() )
		{
		stat = _bios_keybrd(_KEYBRD_SHIFTSTATUS);
		/* Make left and right shift keys equals (bits 0 and 1) and
		 * mask out Caps Lock, Num Lock & Insert (bits 5, 6, and 7).*/
		if (stat&2)   stat |= 1;
		stat &= 0X1D;
		lkey = ((long)_bios_keybrd(_KEYBRD_READ)) + (((long)stat)<<16);
		if (lkey==GIF_C || lkey==GIF_L)
		    {
		    show_last_cmd("SCNSAV");
		    /* Rev 3.50 Removed BMP option */
		    do_gif_file();
		    }
		else
		    break;		/* Key was hit, not screen save */
		}
	    }

	/* Restore display to original & return from function */
	/* Set viewport to entire spectral area */
	setviewport(X1L, Y1T, X2R, Y2B, DO_CLIP);
	/* Put the line data back to top spectral box: */
	for(k=0; k<=128; k++)
	    {
	    putpixel(hpos1, k, *(big_area_ptr1+k));
	    putpixel(hpos2, k, *(big_area_ptr1+1000+k));
	    }
	/* Put the line data back to bottom spectral box: */
	for(k=147; k<=468; k++)
	    {
	    putpixel(hpos1, k, *(big_area_ptr1+k));
	    putpixel(hpos2, k, *(big_area_ptr1+1000+k));
	    }
	/* Replace the screen for the marker data area */
	setviewport(X1L+have-50, 469, X1L+have+50, 479, DO_CLIP);
	putimage(0, 0, big_area_ptr1+2000, COPY_PUT);
	}

/* floppy_door(drv)  -  Checks to see if door is closed
 * floppy drv  (0=A or 1=B). If not A or B, function returns TRUE.
 * If floppy, and door is closed returns TRUE. If floppy and door
 * is open, returns FALSE.
 */
int floppy_door(int drv)
    {
    struct diskinfo_t di;
    unsigned char *dbuf;

    dbuf = (unsigned char *) calloc (2048, sizeof(unsigned char));
    if(dbuf==NULL) return FALSE;

    di.drive=drv;
    di.head=0;
    di.track=1;
    di.sector=1;
    di.nsectors=1;
    di.buffer=dbuf;

    if((drv==0 || drv==1)		/* Are we on A or B drive? */
       && (_bios_disk(_DISK_VERIFY, &di)&0X8000) == 0X8000)  /* Door closed? */
	  {
	  free(dbuf);
          return FALSE;
	  }
    else
	  {
	  free(dbuf);
	  return TRUE;				/* HD or Door closed */
	  }
    }

#if 0
/* Rev 3.50 OUT */
/* Write the entire graphics screen out to a .BMP file */
int do_bmp_file(void)
    {
    #define NUMCOLORS         16  /* Number of different colors */
    #define BITS_PR_PRIM_COLOR 2  /* Two bits per primary color */

    int gpixel(int x, int y);
    int q,                      /* Counter */
	color,                  /* Temporary color value */
	red,                    /* Red component for each color */
	green,                  /* Green component for each color */
	blue;                   /* Blue component for each color */
    struct palettetype pal;
    int itime;
    char ct[15];
    time_t t0;
    struct tm *tgp;
    int drv_num;

    setviewport(0,80,9,90,DO_CLIP);
    clearviewport();
    setcolor(c_text_note);
    settextstyle(0, 0, 1);
    settextjustify(LEFT_TEXT, TOP_TEXT);
    moveto(0,1);
    outtext("*");

    cc[0]=file_ident[0];
    if (ms_available)
	t0=update_clock(1)+dsp_clock_base;
    else
	(void)time(&t0);
    if (time_type)
	    tgp = localtime(&t0);	/* time_type==1 for local */
    else
        tgp = gmtime(&t0);		/*          ==0 for gmt */

    itoa( (1+tgp->tm_mon), ct, 13);   /* Jan=1,..., Dec=C */
    cc[1]=ct[0];
    itoa(tgp->tm_mday, ct, 10);
    if (tgp->tm_mday < 10)
	{
	cc[2]='0';
	cc[3]=ct[0];
	}
    else
	{
	cc[2]=ct[0];
	cc[3]=ct[1];
	}
    /* 100*hour+min will be between 0 and 2359. Need to trap the leading
     * zero cases to prevent a blank from getting to file name.
     */
    itime=100*tgp->tm_hour + tgp->tm_min;
    itoa( itime, ct, 10);
    if (tgp->tm_hour < 10)
	    {
	    cc[4]='0';		/* Less than 0959, so lead with '0' */
	    if (itime>=100)	/* 100 to 999 */
	      {cc[5]=ct[0];  cc[6]=ct[1];  cc[7]=ct[2];}
	    else if (itime>=10)	/* 10 to 99 */
	      {cc[5]='0';  cc[6]=ct[0];  cc[7]=ct[1];}
	    else		/* 0 to 9 */
	      {cc[5]='0';  cc[6]='0';  cc[7]=ct[0];}
	    }
    else
	    {
	    cc[4]=ct[0]; cc[5]=ct[1]; cc[6]=ct[2]; cc[7]=ct[3];
	    }
    cc[8]='.'; cc[9]='B'; cc[10]='M'; cc[11]='P'; cc[12]=0;

    /* If path exists, hang the file name on the end of it */
    if (strlen(fscreen_path)>1)
	{
	strcpy(&cc[100], fscreen_path);	/* Get a cpy to save path string */
	strcat(&cc[100], &cc[0]);	/* Add on path string */
	}
    else
	strcpy(&cc[100], &cc[0]);

    if (cc[100]=='A')        drv_num=0;
    else if (cc[100]=='B')   drv_num=1;
    else                     drv_num=deflt_drv; /* 0, 1, or 999 for HD */

    /* See if floppy is being used and, if so, available */
    if ( !floppy_door(drv_num) )
	{
	do_t_error(51, "E51 Disk Door", 50);
	return FALSE;
	}

    setviewport(0, 0, 639, 479, 1);	/* Needed to start bmp right */

    /* Get the color palette, and extract the red, green and blue
     *  components for each color. In the EGA/VGA palette, colors are
     *  stored as bits in bytes:
     *      00rgbRGB
     *  where r is low intensity red, R is high intensity red, etc.
     *  We shift the bits in place like
     *      000000Rr
     *  for each component to produce 0,1,2,3 level intensities.
     *
     *  BMP colors are '24-bit' and each ranges (0,255). We get these by
     *  multiplying by 85.
     *									*/
    getpalette(&pal);			/* BGI Palette format */
    /* Now convert to .BMP pallette format, store in RGBCColors[][]  */
    for (q = 0; q < NUMCOLORS; q++)
	    {
            color = pal.colors[q];
            red   = ((color & 4) >> 1) | ((color & 32) >> 5);
	    RGBColors[q].red=85*red;
            green = ((color & 2) >> 0) | ((color & 16) >> 4);
	    RGBColors[q].green=85*green;
            blue  = ((color & 1) << 1) | ((color & 8) >> 3);
	    RGBColors[q].blue=85*blue;
	    RGBColors[q].reserved=0;          /* Not used */
	    }
    writebmp(cc+100);

    /* Clear the red '*'   */
    setviewport(0,80,9,90,DO_CLIP);
    clearviewport();
    setcolor(c_text_bold);

    return TRUE;
    }

/* The following is from the work of dotan@corky.net and has been greatly
 * restricted here in only allowing for the 640x480 screen save. See the
 * original bmptest.c for a more general case, although still 16 color.
 * Thanks to dotan@corky.net for the information.
 * Returns TRUE if successful.
 */
int writebmp(char* pictname)
	{
	/* Saves the image within the 640x480 area windows 16 color BMP file */

	BmpHeader      head;
	HeaderInfo     Info;
	int            width;
	unsigned char  c;               /* Used to store pixel data */
	int            i, j;
	unsigned char* scanline;        /* Stores a full line of pixels */
	FILE * fp;

	if (!screen_save) return;

        /* Initialize headers:   */
	head.ImageType[0]='B';
	head.ImageType[1]= 'M';
	head.FileSize = 153718;         /* This and others only for 640x480 */
	head.Reserved1=0;
	head.Reserved2=0;
	head.DataOffset = 118;          /* 14 head + 40 Info + 64 Palette */

	Info.HeaderSize = 40;
	Info.Height = 480;
	Info.NumOfPlanes = 1;
	Info.BitsPerPixel = 4;
	Info.Width = 640;
	width = 320;
	Info.CompMethod = 0;
	Info.BmpSize = 0;               /* Don't really need this data  */
	Info.HorzResolution = 640;
	Info.VertResolution = 480;
	Info.ColorsNumber = 16;
	Info.SigColors = 0;

	fp = fopen(pictname,"wb");
	if(fp==NULL)
	    return TRUE;

        /* Write the headers:    */
	fwrite(&head, sizeof(head), 1, fp);
	fwrite(&Info, sizeof(Info), 1, fp);

	/* Write the pallette (defined above)   */
	fwrite(RGBColors, sizeof(RGBColors[0]), 16, fp);

        /* Scan the 640x480 screen and write the file:  */
	scanline = (unsigned char *) calloc (width+1, sizeof(unsigned char));
	if(scanline==NULL)
		return FALSE;
	for (j=479; j>=0; j--)          /* Bottom to top */
	    {
	    int k=0; i=0;
	    while (i <= 639)
		{
		c = (unsigned char)getpixel(i, j);
		scanline[k] = c<<4;
		c = (unsigned char)getpixel(i+1, j);
		scanline[k] = scanline[k]|c;
		k++;
		i=i+2;                  /* // Two pixels each iteration */
		}
	    /* No end padding required for 640 wide line */
	    if (fwrite(scanline, sizeof(unsigned char), width, fp) != 320)
		{
		free(scanline);
		fclose(fp);
		return FALSE;
		}
	    }
	free(scanline);
	if(fclose(fp) == 0)
	    return TRUE;		/* Close was successful */
	else
	    return FALSE;		/* Close was a problem */
	}
#endif

/* do_gif_file()  -  Records entire graphics screen in a file.
 * The file has a name 'Umddhhmm.GIF' where m is the month (1,C),
 * hhmm is the hour and minute.
 * The 'U' can be changed by file_ident from .CFG file
 * The screen must be in VGA 640x480, 16 colors.
 * Based on S. H. Huseby (gifsave.c)
 */
void do_gif_file(void)
	{
	#define NUMCOLORS         16  /* Number of different colors */
	#define BITS_PR_PRIM_COLOR 2  /* Two bits per primary color */

	int gpixel(int x, int y);

	int q,                      /* Counter */
	    color,                  /* Temporary color value */
	    red[NUMCOLORS],         /* Red component for each color */
	    green[NUMCOLORS],       /* Green component for each color */
	    blue[NUMCOLORS];        /* Blue component for each color */
	struct palettetype pal;
	int ic;
	int itime;
	char ct[15];
	time_t t0;
	struct tm *tgp;
	int drv_num;

	if (!screen_save) return;

	cc[0]=file_ident[0];

	if (ms_available)
	    t0=update_clock(1)+dsp_clock_base;
	else
	    (void)time(&t0);
	if (time_type)
	    tgp = localtime(&t0);	/* time_type==1 for local */
	else
	    tgp = gmtime(&t0);		/*          ==0 for gmt */

	itoa( (1+tgp->tm_mon), ct, 13);   /* Jan=1,..., Dec=C */
	cc[1]=ct[0];
	itoa(tgp->tm_mday, ct, 10);
	if (tgp->tm_mday < 10)
	    {
	    cc[2]='0';
	    cc[3]=ct[0];
	    }
	else
	    {
	    cc[2]=ct[0];
	    cc[3]=ct[1];
	    }
	/* 100*hour+min will be between 0 and 2359. Need to trap the leading
	 * zero cases to prevent a blank from getting to file name.
	 */
	itime=100*tgp->tm_hour + tgp->tm_min;
	itoa( itime, ct, 10);
	if (tgp->tm_hour < 10)
	    {
	    cc[4]='0';		/* Less than 0959, so lead with '0' */
	    if (itime>=100)	/* 100 to 999 */
	      {cc[5]=ct[0];  cc[6]=ct[1];  cc[7]=ct[2];}
	    else if (itime>=10)	/* 10 to 99 */
	      {cc[5]='0';  cc[6]=ct[0];  cc[7]=ct[1];}
	    else		/* 0 to 9 */
	      {cc[5]='0';  cc[6]='0';  cc[7]=ct[0];}
	    }
	else
	    {
	    cc[4]=ct[0]; cc[5]=ct[1]; cc[6]=ct[2]; cc[7]=ct[3];
	    }
	cc[8]='.'; cc[9]='G'; cc[10]='I'; cc[11]='F'; cc[12]=0;

	/* If path exists, hang the file name on the end of it.
	 * Full file name for opening ends up at &cc[100]
	 */
	if (strlen(fscreen_path)>1)
	    {
	    strcpy(&cc[100], fscreen_path);
	    strcat(&cc[100], &cc[0]);
	    }
	else
	    strcpy(&cc[100], &cc[0]);

	/* deflt_drv comes from DOS at startup */
	if (cc[100]=='A')        drv_num=0;
	else if (cc[100]=='B')   drv_num=1;
	else                     drv_num=deflt_drv; /* 0, 1, or 999 for HD */

        /* See if floppy is being used and, if so, available */
        if ( !floppy_door(drv_num) )
	    {
	    do_t_error(51, "E51 Disk Door", 50);
	    return;
	    }

	setviewport(0,80,9,90,DO_CLIP);
	clearviewport();
	setcolor(c_text_note);
	settextstyle(0, 0, 1);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(0,1);
	outtext("*");

	setviewport(0, 0, 639, 479, 1);	/* Needed to start gif right */

	/* Get the color palette, and extract the red, green and blue
 	*  components for each color. In the EGA/VGA palette, colors are
	*  stored as bits in bytes:
	*      00rgbRGB
	*  where r is low intensity red, R is high intensity red, etc.
	*  We shift the bits in place like
	*      000000Rr
	*  for each component					*/
	getpalette(&pal);
	for (q = 0; q < NUMCOLORS; q++)
	    {
            color = pal.colors[q];
            red[q]   = ((color & 4) >> 1) | ((color & 32) >> 5);
            green[q] = ((color & 2) >> 0) | ((color & 16) >> 4);
            blue[q]  = ((color & 1) << 1) | ((color & 8) >> 3);
	    }

	GIF_Create(&cc[100], 640, 480, NUMCOLORS, BITS_PR_PRIM_COLOR);

	/*  Set each color according to the values extracted from
	 *  the palette    */
	for (q = 0; q < NUMCOLORS; q++)
	GIF_SetColor(q, red[q], green[q], blue[q]);

	/*  Store the entire screen as an image using the user defined
	 *  callback function gpixel() to get pixel values from the screen */
	GIF_CompressImage(0, 0, -1, -1, gpixel);

	GIF_Close();

	setviewport(0,80,9,90,DO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	}

int gpixel(int x, int y)
	{
	return getpixel(x, y);
	}

/* print_big(ch, pos, color) prints a double size character in the top
 * box.  The position can be specified as 0 to 30, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * Available in waterfall configuration, only. No backspace.
 */
void print_big(char ch, int pos, int color)
	{static int print_pos=0;
	 static char c[31];
	 static char c1[2];
	 int i;

	 if (pos==-1)			/* Initialize data */
	    {for(i=0; i<30; i++)  c[i]=32;
	     c[30]=0;
	     print_pos=0;
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==30)		/* In scroll position */
		{for (i=0; i<29; i++)  c[i]=c[i+1];
		 c[29]=ch;
		}
	     else			/* Not yet scrolling */
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;
	    c1[0]=ch; c1[1]=0;
	    setviewport(132+16*pos, 13, 148+16*pos, 32, DO_CLIP);
	    clearviewport();
	    setcolor(color);
	    settextstyle(0, 0, 2);
	    settextjustify(0,2);
	    moveto(0,2);
	    outtext(c1);
	    setcolor(c_text_bold);
	    return;
	    }
	 setviewport(129, 13, 638, 32, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 2);
	 settextjustify(0,2);
	 moveto(3,2);
	 outtext(c);
	}

/* print_super_row(ch, pos, color) prints a in the top box, centered over
 * the character from print_big(...)
 * The position can be specified as 0 to 30. If pos=-1 then initialize.
 * Available in waterfall configuration, only. No backspace.
 */
void print_super_row(char ch, int pos, int color)
	{
	static char c1[2];

	if (pos==-1)			/* Initialize data */
	    {
	    c1[0]=0; c1[1]=0;
	    setviewport(132, 2, 628, 12, DO_CLIP);
	    clearviewport();
	    }
	else
	    {
	    c1[0]=ch; c1[1]=0;
	    setviewport(135+16*pos, 2, 146+16*pos, 12, DO_CLIP);
	    clearviewport();
	    setcolor(color);
	    settextstyle(0, 0, 1);
	    settextjustify(0,2);
	    moveto(0,1);
	    outtext(c1);
	    setcolor(c_text_bold);
	    return;
	    }
	}

/* print_medium(ch, pos) prints a standard size character in the top
 * box.  The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 */
void print_medium(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[3*LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<3*LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[3*LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==254)		/* Code for backspace */
	    {
	    if (print_pos>0)
		{
		c[--print_pos] = 32;		/* Erase last char */
		}
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==3*LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<3*LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[3*LINE_LEN-2]=ch;
		}
	     else
		{
		c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;			/* Random access to pos */
	    }
	 if (waterfall)
	     setviewport(129, 1, 638, 33, DO_CLIP);
	 else
	     setviewport(X2L+1, Y2T+59, X2R-1, Y2T+98, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);
	 moveto(3, 2);
	 save_ch = c[LINE_LEN];  c[LINE_LEN] = 0;
	 outtext(c);
	 c[LINE_LEN] = save_ch;
	 save_ch = c[2*LINE_LEN];  c[2*LINE_LEN] = 0;
	 moveto(3, 12);
	 outtext(&c[LINE_LEN]);
	 c[2*LINE_LEN] = save_ch;
	 moveto(3, 21);
	 outtext(&c[2*LINE_LEN]);
	}

/* print_medium_1(ch, pos) prints a standard size character in the top
 * box.  Single line at top ONLY.
 * The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * Also responds to pos=254 and does a backspace.
 * Available in waterfall configuration, only.
 */
void print_medium_1(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==254)		/* Code for backspace */
	    {
	    if (print_pos>0)
	    c[--print_pos] = 32;	/* Erase last char */
	    }
	 else if(pos==255)		/* In scroll mode */

	    {if (print_pos==LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[LINE_LEN-2]=ch;
		}
	     else
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;			/* Random access to position */
	    }

	 setviewport(129, 1, 638, 11, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);
	 moveto(3, 2);
	 outtext(c);
	}

/* print_medium_2(ch, pos) prints a standard size character in the top
 * box.  This one prints 2 line, in the bottom 2/3 of the box.
 * The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * No backspace, available in waterfall configuration, only.
 */
void print_medium_2(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[2*LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<2*LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[2*LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==2*LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<2*LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[2*LINE_LEN-2]=ch;
		}
	     else
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;
	    }

	 setviewport(129, 12, 638, 33, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);

	 moveto(3, 2);
	 save_ch = c[LINE_LEN];  c[LINE_LEN] = 0;
	 outtext(c);
	 c[LINE_LEN] = save_ch;

	 moveto(3, 12);
	 outtext(&c[LINE_LEN]);
	}

/* call2dos()  -  Allows running of a new DOS control and a return with
 * 'exit'.  Exits the graphics mode after saving the screen. Also updates the
 * UHFA.CFG file and re-reads this upon reentry to DSP-10.  Proper care is
 * taken to restore the drive and path of the DSP-10.
 */
void call2dos(void)
    {
    int grdrv, grmode;
    int mem_err;
    char cpath[128];
    char cdrv[3];

    save_state();			/* Write UHFA.CFG */

    setviewport(129, 128, 639, 479, DO_CLIP);
    getimage(0,  0, 510, 191, big_area_ptr1);
    getimage(0, 192, 510, 351, big_area_ptr2);

    setallpalette(&savpalette);	/* The palette when we got here */
    setbkcolor(BLACK);
    setcolor(WHITE);
    closegraph();

    printf("\nThis is a new copy of the DOS command processor.\n");
    printf("Any edit to UHFA.CFG will be included after return to DSP-10.\n");
    printf("Avoid changing .CFG parameters that are");
    printf(" available from the DSP-10 keyboard.\n");
    printf("The DSP-10 still resides in memory. Type 'Exit' to resume\n");
    printf("radio operation. Forgetting this may cause problems!\n");

    /* Find the current directory path, to return to later */
    getcwd(cpath, 128);
    /* In case the drive is changed, we need to be able to restore this */
    cdrv[0]=cpath[0];  cdrv[1]=cpath[1];  cdrv[2]=(char)0;

    system("command");			/* Opens a new command.com */

    /* Return to here after 'Exit' of Dos */

    system(cdrv);			/* Reset drive in case of change */
    chdir(cpath);			/* Restore old path on that drive */

    grdrv=9;				/* Only allow VGA 640x480 */
    grmode=2;
    initgraph(&grdrv, &grmode, "");

    getpalette(&savpalette);	/* Palette when we arrived */
    setallpalette(&upalette);
    if ( graphresult()==grError )
	do_t_error(40, "E40 Graphics", 200);

    setbkcolor(c_background);
    draw_grid();
    setviewport(129, 128, 639, 479, DO_CLIP);
    putimage(0,   0, big_area_ptr1, COPY_PUT);
    putimage(0, 192, big_area_ptr2, COPY_PUT);

    if (read_config_file() == 0)	/* Get data from UHFA.CFG */
	{
	printf("\nNo uhfa.cfg present, using pre-DOS call values");
	}
    initialize(2);			/* Restrict to updating controls */
    }

void dsp_cmd(int b1, int b2, int b3, int b4, int b5)
	{
	ASYNCOUT(ESC_CHAR);
	ASYNCOUT(b1 & 0XFF);
	ASYNCOUT(b2 & 0XFF);
	ASYNCOUT(b3 & 0XFF);
	ASYNCOUT(b4 & 0XFF);
	ASYNCOUT(b5 & 0XFF);
	}

void show_last_cmd(char* ct)
	{
	setviewport(40, 470, 89, 479, DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(25, 0);
	setcolor(c_text);
	outtext(ct);
	setcolor(c_text_bold);
	}

/* show_beacon()  -  Display the beacon parameters available in LTI.
 * these go to the right edge of the upper spectral display box.
 * Made separate function RSL 29 Jan 03
 */
void show_beacon(void)
    {
    setviewport(X1R-62, Y1T+85, X1R-3, Y1T+94, DO_CLIP);
    clearviewport();
    settextjustify(CENTER_TEXT,TOP_TEXT);
    setcolor(c_text_bold);
    moveto(30, 0);
    outtext("BEACON");

    if(b_transmit_sec>999 || b_receive_sec>999)
	{
	setviewport(X1R-78, Y1T+101, X1R-3, Y1T+110, DO_CLIP);
	clearviewport();
	moveto(38, 0);
	}
    else			/* Need less width */
	{
	setviewport(X1R-62, Y1T+101, X1R-3, Y1T+110, DO_CLIP);
	clearviewport();
	moveto(30, 0);
	}
    sprintf(cc, "%d %d", b_transmit_sec, b_receive_sec);
    outtext(cc);

    setviewport(X1R-62, Y1T+117, X1R-3, Y1T+126, DO_CLIP);
    clearviewport();
    moveto(30, 0);
    if (mode==LTI && lt_b_save && lt_b_clear)
	sprintf(cc, "S%dC", b_offset_sec);
    else if (mode==LTI && lt_b_save && !lt_b_clear)
	sprintf(cc, "S%d", b_offset_sec);
    else if (mode==LTI && !lt_b_save && lt_b_clear)
	sprintf(cc, "%dC", b_offset_sec);
    else
	sprintf(cc, "%d", b_offset_sec);
    outtext(cc);

    /* Show the active times of the hour, if in use. Above "BEACON."  */
    if(b_active1>0 || b_active2>0)
	{
	setviewport(X1R-58, Y1T+72, X1R-1, Y1T+80, DO_CLIP);
	clearviewport();
	tiny_int(b_active1, 0, 0);
	tiny_int(b_active2, 30, 0);
	}
    }

#if 0
/* show_noise_power()  -  Display the power parameters available in FM.
 */
void show_noise power(void)
    {
    setviewport(X1R-62, Y1T+85, X1R-3, Y1T+94, DO_CLIP);
    clearviewport();
    settextjustify(CENTER_TEXT,TOP_TEXT);
    setcolor(c_text_bold);
    moveto(30, 0);
    outtext("BEACON");

    if(b_transmit_sec>999 || b_receive_sec>999)
	{
	setviewport(X1R-78, Y1T+101, X1R-3, Y1T+110, DO_CLIP);
	clearviewport();
	moveto(38, 0);
	}
    else			/* Need less width */
	{
	setviewport(X1R-62, Y1T+101, X1R-3, Y1T+110, DO_CLIP);
	clearviewport();
	moveto(30, 0);
	}
    sprintf(cc, "%d %d", b_transmit_sec, b_receive_sec);
    outtext(cc);

    setviewport(X1R-62, Y1T+117, X1R-3, Y1T+126, DO_CLIP);
    clearviewport();
    moveto(30, 0);
    if (mode==LTI && lt_b_save && lt_b_clear)
	sprintf(cc, "S%dC", b_offset_sec);
    else if (mode==LTI && lt_b_save && !lt_b_clear)
	sprintf(cc, "S%d", b_offset_sec);
    else if (mode==LTI && !lt_b_save && lt_b_clear)
	sprintf(cc, "%dC", b_offset_sec);
    else
	sprintf(cc, "%d", b_offset_sec);
    outtext(cc);

    /* Show the active times of the hour, if in use. Above "BEACON."  */
    if(b_active1>0 || b_active2>0)
	{
	setviewport(X1R-58, Y1T+72, X1R-1, Y1T+80, DO_CLIP);
	clearviewport();
	tiny_int(b_active1, 0, 0);
	tiny_int(b_active2, 30, 0);
	}
    }
#endif

/* left_justify(char *str1, char *str2, int len2, char fill)
 * str1 is copied to str2 and the space to the right is filled with spaces.
 * Therefore str1 should not be longer than len2 or it will be truncated.
 * The difference between str1 and str2 is right-filled with 'fill'.
 * str2 is terminated with a null and must be declared as len2+1 or more.
 */
char *left_justify(char *str1, char *str2, int len2, char fill)
    {
    int len1, l, i;

    len1=strlen(str1);
    if (len1>len2)			/* Won't fit case!  */
	l=len2;
    else
	{
	l=len1;
	for (i=len1; i<len2; i++)	/* Fill to end with blanks */
	    *(str2+i)=fill;
	}
    for (i=0; i<l; i++)			/* Move the string into place */
	*(str2+i)=*(str1+i);
    if (l < len2)
	*(str2+l)=32;			/* Single blank at end; readability */
    *(str2+len2)=0;			/* Terminate the string in a null */
    return str2;
    }

void delayu(unsigned int ncount)
	{unsigned int i;
	 for (i=0; i<ncount; i++);
	}

/* Alternate error handlers  so that program does not exit to dos from
   Ctrl-C or from floating point error */
void  ctrlc_handler(int sig)
	{
	do_t_error(45, "W45 Ctrl-C", 0);
	signal(SIGINT, ctrlc_handler);  /* Reinstall for next time */
	}
void  fpe_handler(int sig)
	{
	errcode=_clear87();
	do_t_error(46,"E46 Float Point", 0);
	signal(SIGFPE, fpe_handler);
	}

void display_summary(char *fname, char *ver)
	{
	char  *pc;

	strcpy(cc, fname);		/* Get a copy to play with */
	cc[strlen(cc)-4]=0;		/* Remove the .exe on the end */
	pc=&cc[strlen(cc)];		/* Now look for end of path */
	while (*pc!='\\' && pc!=&cc[0])  pc--;
	pc++;				/* Move past the slash */
	printf("\nUHFA Transceiver Controller v%s  de W7PUA\n\n", ver);
	printf("Your command line lacked necessary information.\n");
	printf("Use '%s com' where com is a number between 1 and\n", pc);
	printf("4 that sets the COM port to be used.\n");
	printf("Thus for COM2 enter\n");
	printf("        %s 2 \n\n", pc);
	}

void point1(int x, int y)	/* Cross */
	{
	line(x, y-1, x, y+1);
	line(x-1, y, x+1, y);
	}

void point5(int x, int y)	/* Block rectangle */
	{
	line(x-1, y-1, x+2, y-1);
	line(x-1, y-1, x-1, y+1);
	line(x+2, y-1, x+2, y+1);
	line(x-1, y+1, x+2, y+1);
	}

/* tiny_int(n, x0, y0, justify) puts n into an area with U.L. coords x0, y0.
 * The number is left justified from the first significant digit.
 * Only positive numbers are handled.   RSL 1 Feb 03.
 */
void tiny_int(int n, int x0, int y0)
	{
	int d[5], k, len_d;

	if (n<0)
	    {
	    line(x0+6, y0+3, x0+10, y0+3);
	    return;
	    }
	/* Find individual digits with MSD in d[1]  */
	d[1]=n/10000;
	n-=d[1]*10000;
	d[2]=n/1000;
	n-=d[2]*1000;
	d[3]=n/100;
	n-=d[3]*100;
	d[4]=n/10;
	n-=d[4]*10;
	d[5]=n;
	len_d=5;

	/* Now drop off leading zeros */
	if(d[1]==0)
	    {
	    for(k=1; k<=4; k++)  d[k]=d[k+1];
	    len_d--;
	    }
	if(d[1]==0)
	    {
	    for(k=1; k<=3; k++)  d[k]=d[k+1];
	    len_d--;
	    }
	if(d[1]==0)
	    {
	    for(k=1; k<=2; k++)  d[k]=d[k+1];
	    len_d--;
	    }
	if(d[1]==0)
	    {
	    d[1]=d[2];
	    len_d--;
	    }
	for(k=1; k<=len_d; k++)
	    tiny_char(d[k], x0+6*(k-1), y0+8);
	}

/* tiny_time( ) puts the time along the left side of the spectral map.
 * The area 9 high and 32 wide should be cleared.
 * If show_secs==TRUE there is a colon between the first two and the
 * last two digits.
 */
void tiny_time(int hhmm, int x0, int y0)
	{
	int h1, h2, m1, m2;
	h1=hhmm/1000;
	hhmm-=h1*1000;
	h2=hhmm/100;
	hhmm-=h2*100;
	m1=hhmm/10;
	m2=hhmm-m1*10;
	tiny_char(h1, x0, y0);
	tiny_char(h2, x0+6, y0);
	if (show_secs)
	    {
	    putpixel(x0+13, y0-1, c_text_bold);
	    putpixel(x0+13, y0-4, c_text_bold);
	    tiny_char(m1, x0+15, y0);
	    tiny_char(m2, x0+21, y0);
	    line(x0+27, y0, x0+33, y0);	/* Tick for actual time */
	    }
	else
	    {
	    tiny_char(m1, x0+12, y0);
	    tiny_char(m2, x0+18, y0);
	    line(x0+24, y0, x0+30, y0);	/* Tick for actual time */
	    }
	}

/* Print n with bottom left at (x0, y0) */
void tiny_char(int n, int x0, int y0)
	{
	if (n<0 || n>9) return;
	switch (n)
	  {
	  case 0:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-6, x0+4, y0-6);
		line(x0, y0, x0, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  case 1:{
		line(x0+2, y0, x0+2, y0-6);
		break;
		}
	  case 2:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-6, x0+4, y0-6);
		line(x0, y0, x0, y0-3);
		line(x0+4, y0-3, x0+4, y0-6);
		break;
		}
	  case 3:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-6, x0+4, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  case 4:{
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-3, x0, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  case 5:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-6, x0+4, y0-6);
		line(x0+4, y0, x0+4, y0-3);
		line(x0, y0-3, x0, y0-6);
		break;
		}
	  case 6:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0, x0, y0-6);
		line(x0+4, y0, x0+4, y0-3);
		break;
		}
	  case 7:{
		line(x0, y0-6, x0+4, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  case 8:{
		line(x0, y0, x0+4, y0);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-6, x0+4, y0-6);
		line(x0, y0, x0, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  case 9:{
		line(x0, y0-3, x0, y0-6);
		line(x0, y0-3, x0+4, y0-3);
		line(x0, y0-6, x0+4, y0-6);
		line(x0+4, y0, x0+4, y0-6);
		break;
		}
	  }
	}

/* Print smaller n with bottom left at (x0, y0). 3 Wide, 4 high */
void micro_char(int n, int x0, int y0)
	{
	if (n<0 || n>9) return;
	switch (n)
	  {
	  case 0:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-4, x0+2, y0-4);
		line(x0, y0, x0, y0-4);
		line(x0+2, y0, x0+2, y0-4);
		break;
		}
	  case 1:{
		line(x0+1, y0, x0+1, y0-4);
		break;
		}
	  case 2:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-1, x0+2, y0-3);
		line(x0, y0-4, x0+2, y0-4);
		break;
		}
	  case 3:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0-4, x0+2, y0-4);
		line(x0+2, y0, x0+2, y0-4);
		break;
		}
	  case 4:{
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0-2, x0, y0-4);
		line(x0+2, y0, x0+2, y0-4);
		break;
		}
	  case 5:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0-4, x0+2, y0-4);
		line(x0+2, y0-1, x0+2, y0-2);
		line(x0, y0-2, x0, y0-4);
		break;
		}
	  case 6:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0, x0, y0-2);
		line(x0, y0-2, x0+1, y0-4);
		line(x0+2, y0, x0+2, y0-2);
		break;
		}
	  case 7:{
		line(x0, y0-4, x0+2, y0-4);
		line(x0+1, y0, x0+2, y0-4);
		break;
		}
	  case 8:{
		line(x0, y0, x0+2, y0);
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0-4, x0+2, y0-4);
		line(x0, y0, x0, y0-4);
		line(x0+2, y0, x0+2, y0-4);
		break;
		}
	  case 9:{
		line(x0, y0-2, x0, y0-4);
		line(x0, y0-2, x0+2, y0-2);
		line(x0, y0-4, x0+2, y0-4);
		line(x0+2, y0, x0+2, y0-4);
		break;
		}
	  }
	}

/* Draw bar graph, starting from right bar. If value smaller than test
 * value, just draw an empty bar. When smaller, continue to fill in all bars.
 * Style:  0   5 White, 5 Green, 5 Red
 *         1   5 White, 9 Green, 1 Red
 *         2   5 White, 5 Green, 5 Red with test value x2, Plus 20 pixels
 *                   of overload marker if bit 14 of value is a 1.
 * value is magnitude of an int, normally, and 15 bits (32767) maximum.
 */
void bar_graph(int x0, int y0, unsigned int value, int style)
	{
	int i, color_on;
	unsigned int test_value;

	if (style==2)
	    {
	    if (value & 0X4000)
		{
		setfillstyle(SOLID_FILL, bar_color[14]);
		bar(x0+91, y0, x0+109, y0+8);	/* Red center */
		setcolor(bar_color[0]);		/* 2 white outlines */
		line(x0+90, y0, x0+90, y0+8);
		line(x0+110, y0, x0+110, y0+8);
		value = 0X3FFF;	/* Force all bars red during overload */
		}
	    value = (value & 0X3FFF) * 2;
	    }
	setfillstyle(SOLID_FILL, c_bar_dark);   /* Unlit bar color */
	test_value = 0X4000;
	for(i=14; i>=0; i--)			/* Start from right */
		{
		color_on = ( value >= test_value );
		test_value /= 2;
		if (color_on && (style==0 || style==2))	/* 6 red bars */
			setfillstyle(SOLID_FILL, bar_color[i]);
		else if (color_on && style==1)	/* 1 red bar - Overload mark */
			setfillstyle(SOLID_FILL, bar_color_o[i]);
		bar(x0+i*6, y0, x0+i*6+4, y0+8);
		}
	}

/*************************************************************************
 * A modal dialog box for exiting DSP-10.
 * RSL 5 Jan 04
 */
int exit_modalbox(void)
	{
	static struct edit_info   el;
	char eline[24];
	unsigned int edresult;
	struct lbox_info  *lb_ptr;

	if ( (lb_ptr = (void *)malloc(sizeof(struct lbox_info)) ) == NULL)
		{
		do_t_error(54, "E54 Exit box", 50);
		return FALSE;
		}
 
	lb_create(lb_ptr, "EXIT ?", 250, 282, 438, 357, 0, MODAL);
	lb_open(lb_ptr, 1);

	/* There are 6 lines, all need to be declared lb_line()'ed */
	/* Fixed text in modal box:     */
	lb_line(lb_ptr, 0, BLANK, "", 0);
	lb_line(lb_ptr, 1, TXT, "   Y <Enter> to Exit", 0);
	lb_line(lb_ptr, 2, TXT, " Any other key Cancels", 0);
	lb_line(lb_ptr, 3, BLANK, "", 0);
	/* An edit field for getting a character */
	eline[0]=0;			/* An empty field */
	lb_create_el(&el, 2, 70, &eline[0], ED_ACTIVE, 0, 0);
	lb_line(lb_ptr, 4, EDIT, "", &el);
	lb_line(lb_ptr, 5, BLANK, "", 0);

	lb_update(lb_ptr);

	/* Following is a loop for handling user keyboard input */
 e0:	edresult=lb_edit_line(lb_ptr, &el);

	/* Close box, then screen save */
	/* If the comment lines are empty, we will remove the comment box
	 * and save the screen.
	 */
	free(lb_ptr);
	if(eline[0]=='y' || eline[0]=='Y')
		return 1;
	lb_close(lb_ptr, 1);
	return 0;
	}

/*************************************************************************
 * A modal dialog box for "beacon" parameters.
 * The box structure is lb_ut1.
 */
void beacon_modalbox(void)
	{
	static struct edit_info   el1, el2, el3, el4;
	static struct check_info  ck1, ck2;
	char eline1[12], eline2[12], eline3[12], eline4[12];
	unsigned int edresult, ckui;
	int sl;
	struct lbox_info  *lb_ut1_ptr;

	if ((lb_ut1_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 
	lb_create(lb_ut1_ptr, "BEACON PARAMETERS",
			 160, 175, 370, 440, 0, MODAL);
	lb_open(lb_ut1_ptr, 1);

	/* Fixed text in modal box:     */
	lb_line(lb_ut1_ptr, 21, BOLD, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut1_ptr, 22, BOLD, "SPACE changes buttons", 0);
	lb_line(lb_ut1_ptr, 23, BOLD, "\030 and \031 Move to Next Item", 0);
	lb_line(lb_ut1_ptr, 24, BOLD, "ENTER=Close    ESC=Cancel", 0);

	/* An edit field for Transmit time: */
	sprintf(eline1,"%d", b_transmit_sec); /* The field to be edited */
	sl=strlen(eline1);
	lb_create_el(&el1, 7, 135, &eline1[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut1_ptr, 4, EDIT, "Transmit seconds", &el1);

	/* An edit field for Receive time: */
	sprintf(eline2,"%d", b_receive_sec); /* The field to be edited */
	sl=strlen(eline2);
	lb_create_el(&el2, 7, 135, &eline2[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut1_ptr, 6, EDIT, "Receive seconds", &el2);

	/* An edit field for Offset time: */
	sprintf(eline3,"%d", b_offset_sec); /* The field to be edited */
	sl=strlen(eline3);
	lb_create_el(&el3, 7, 135, &eline3[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut1_ptr, 8, EDIT, "Offset seconds", &el3);

	if (b_repeat60)
	    lb_line(lb_ut1_ptr, 16, TXT, "Last time per 60 min:", 0);
	else
	    lb_line(lb_ut1_ptr, 16, TXT, "Last time per 30 min:", 0);

	/* An edit field for Beacon active stop time: */
	sprintf(eline4, "%d", b_active2); /* The field to be edited */
	sl=strlen(eline4);
	lb_create_el(&el4, 7, 135, &eline4[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut1_ptr, 17, EDIT, "Beacon Stop secs", &el4);

	/* Parameters in call to lb_create_check(....)
	 *   struct check_info    One data structure per check mark
	 *   int                  Check position in pixels from left edge
	 *   unsigned int         State of check, checked=TRUE
	 *   int                  Text position in pixels from left edge
	 */

	/* Save LT data to file xxxxxx.DAT check box:     */
	if (lt_b_save)			/* Show box checked */
	    lb_create_check(&ck1, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck1, 0, FALSE, 20);
	lb_line(lb_ut1_ptr, 12, CHECK, "Save LT Data to Disk", &ck1);

	/* Clear LT data after save check box:     */
	if (lt_b_clear)			/* Show box checked */
	    lb_create_check(&ck2, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck2, 0, FALSE, 20);
	lb_line(lb_ut1_ptr, 14, CHECK, "Clear LT Dat after Sav", &ck2);

	lb_update(lb_ut1_ptr);

	/* Following is a loop for handling user keyboard input */
 e4:	edresult=lb_edit_line(lb_ut1_ptr, &el1);
	if (edresult==CUR_UP)   goto e17;
	else if (edresult==ENTER) goto lb_ut1end;

 e6:	edresult=lb_edit_line(lb_ut1_ptr, &el2);
	if (edresult==CUR_UP)   goto e4;
	else if (edresult==ENTER) goto lb_ut1end;

 e8:	edresult=lb_edit_line(lb_ut1_ptr, &el3);
	if (edresult==CUR_UP)   goto e6;
	else if (edresult==ENTER) goto lb_ut1end;

 c12:	ckui=lb_check_mark(lb_ut1_ptr, &ck1);
	if     (ckui==CUR_UP)  goto e8;
	else if(ckui==ENTER)     goto lb_ut1end;

 c14:	ckui=lb_check_mark(lb_ut1_ptr, &ck2);
	if     (ckui==CUR_UP)  goto c12;
	else if(ckui==ENTER)     goto lb_ut1end;

 e17:	edresult=lb_edit_line(lb_ut1_ptr, &el4);
	if (edresult==CUR_UP)   goto c14;
	else if (edresult==ENTER) goto lb_ut1end;

	goto e4;

 lb_ut1end: delay(200);
	/* As we exit, the variables must be changed to agree with the
	 * indicated states of the buttons and edit fields:
	 */
	b_transmit_sec=atoi(eline1);	/* 1 or 2 is illegal */
	if (b_transmit_sec<3)   b_transmit_sec=0;

	b_receive_sec=atoi(eline2);
	if (b_receive_sec<0)   b_receive_sec=0;

	b_offset_sec=atoi(eline3);
	if (b_offset_sec<0)   b_offset_sec=0;

	if (ck1.state & CHECK_MARK)
	    lt_b_save=TRUE;
	else
	    lt_b_save=FALSE;

	if (ck2.state & CHECK_MARK)
	    lt_b_clear=TRUE;
	else
	    lt_b_clear=FALSE;

	b_active2=atoi(eline4);
	if (b_active2<0)   b_active2=0;

	lb_close(lb_ut1_ptr, 1);
	free(lb_ut1_ptr);
	}

/*************************************************************************
 * A modal dialog box for screen save comments.
 * The box structure is lb_ut2.
 * RSL 9 Feb 03
 */
void scs_modalbox(void)
	{
	static struct edit_info   el1, el2;
	char eline1[24], eline2[24];
	unsigned int edresult;
	struct lbox_info  *lb_ut2_ptr;

	if ( (lb_ut2_ptr = (void *)malloc(sizeof(struct lbox_info)) ) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 
	lb_create(lb_ut2_ptr, "COMMENTS", 450, 382, 638, 457, 0, MODAL);
	lb_open(lb_ut2_ptr, 1);

	/* Fixed text in modal box:     */
	lb_line(lb_ut2_ptr, 4, TXT, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut2_ptr, 5, TXT, "ENTER=Close box & Save", 0);

	/* An edit field for comment1 */
	eline1[0]=0;			/* An empty field */
	lb_create_el(&el1, 22, 0, &eline1[0], ED_ACTIVE, 0, 0);
	lb_line(lb_ut2_ptr, 0, EDIT, "", &el1);

	/* An edit field for comment2 */
	eline2[0]=0;			/* An empty field */
	lb_create_el(&el2, 22, 0, &eline2[0], ED_INACTIVE, 0, 0);
	lb_line(lb_ut2_ptr, 2, EDIT, "", &el2);

	lb_line(lb_ut2_ptr, 1, BLANK, "", &el2);
	lb_line(lb_ut2_ptr, 3, BLANK, "", &el2);
	lb_update(lb_ut2_ptr);

	/* Following is a loop for handling user keyboard input */
 e0:	edresult=lb_edit_line(lb_ut2_ptr, &el1);
	if (edresult==CUR_UP)   goto e2;
	else if (edresult==ENTER) goto lb_ut2end;

 e2:	edresult=lb_edit_line(lb_ut2_ptr, &el2);
	if (edresult==CUR_UP)   goto e0;
	else if (edresult==ENTER) goto lb_ut2end;
	goto e0;

 lb_ut2end:				/* Close box, then screen save */
	/* If the comment lines are empty, we will remove the comment box
	 * and save the screen.
	 */
	if(strlen(eline1)==0 && strlen(eline2)==0)
	    {
	    lb_close(lb_ut2_ptr, 1);
	    free(lb_ut2_ptr);
	    /* Rev 3.50 Removed BMP option */	
	    do_gif_file();
	    return;
	    }
	else
	    { /* Redraw box, Screen save, then close box */
	    /* To reduce confusion, we will remove the modal-box instructions
	     * by closing the box and creating a new box, 2-lines shorter.
	     * Strings eline1[] and eline2[] already have the comments.
	     * While we're at it, we can brighten up both lines to ED_ACTIVE.
	     */
	    lb_close(lb_ut2_ptr, 1);

	    lb_create(lb_ut2_ptr, "COMMENTS", 450, 382, 638, 417, 0, MODAL);
	    lb_open(lb_ut2_ptr, 1);
	    /* Comments from before:     */
	    lb_line(lb_ut2_ptr, 0, BOLD, &eline1[0], 0);
	    lb_line(lb_ut2_ptr, 1, BOLD, &eline2[0], 0);
	    lb_update(lb_ut2_ptr);
	    /* Rev 3.50 Removed BMP option */
	    do_gif_file();
	    lb_close(lb_ut2_ptr, 1);
	    free(lb_ut2_ptr);
	    }
	}

/* scs_edit_line(...)
 * Version of lb_edit_line that allows ALT-G return for screen saving
 * Returns codes, depending on exit character:
 *    Up Cursor:   0X4800
 *    Down Cursor: 0X5000
 *    ENTER:       0X000D
 *    ALT-G:       0X2200
 */
unsigned int scs_edit_line(struct lbox_info* lbi, struct edit_info* el)
	{
	unsigned int c, cl, ch, r;
	int          cli;

	lb_edit_state(el, ED_ACTIVE);
	lb_update(lbi);			/* Make display current */
	while (1)
	   {
	    if ( kbhit() )
		{
		c = _bios_keybrd(_KEYBRD_READ);
		ch=c&0XFF00;
		cl=c&0X00FF;   cli=(int)cl;

		if (cl==ENTER)			/* Enter */
		   {r=cl;
		    break;
		   }
		else if (ch==CUR_UP)		/* Up Cursor */
		   {r=ch;
		    break;
		   }
		else if (ch==CUR_DOWN)		/* Down Cursor */
		   {r=ch;
		    break;
		   }
		else if (c==0X2200)		/* ALT-G */
		   {r=c;
		    break;
		   }
		else if (cl==BACKSPACE && el->cursor_pos>0)  /* Backspace */
		    {
		    el->cursor_pos--;
		    *(el->edit_line + el->cursor_pos) = 0; /* New EOL */
		    lb_update(lbi);
		    }
		else if ( (el->cursor_pos < el->edit_line_nch-1)
			    && (isgraph(cli) || cli==' ') )
		    {
		    *(el->edit_line + el->cursor_pos) = (char)cl;
		    el->cursor_pos++;
		    *(el->edit_line + el->cursor_pos) = 0; /* New EOL */
		    lb_update(lbi);
		    }

		}     /* End if kbhit() */
	    }         /* End while(1) */
	lb_edit_state(el, ED_INACTIVE);
	return r;
	}

/*************************************************************************
 * A modal dialog box for V2F Tone parameters.
 * The box structure is lb_ut3.
 */
void v2f_modalbox(void)
	{
	static struct edit_info   el1, el2, el3, el4, el5, el6;
	static struct radio_info  ra1, ra2;
	char eline1[12], eline2[12], eline3[12], eline4[12],
	    eline5[12], eline6[12];
	unsigned int edresult, rbui, bpush;
	int sl;
	double bin, half_bin, vdf, oldf, oldf2;
	struct lbox_info  *lb_ut3_ptr;
	int do_delay, oldi;
	/* Dynamic allocation of memory for box data */
	if ((lb_ut3_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 
	lb_create(lb_ut3_ptr, "V2F Tone Parameters",
			 160, 175, 370, 440, 0, MODAL);
	lb_open(lb_ut3_ptr, 1);

	/* Fixed text in modal box:     */
	lb_line(lb_ut3_ptr, 21, BOLD, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut3_ptr, 22, BOLD, "SPACE changes buttons", 0);
	lb_line(lb_ut3_ptr, 23, BOLD, "\030 and \031 Move to Next Item", 0);
	lb_line(lb_ut3_ptr, 24, BOLD, "ENTER=Close    ESC=Cancel", 0);

	if (v2f_noise) bpush=1;   else bpush=2;
	lb_create_rb(&ra1, 0, bpush, 20);	/* Note: bpush pushed */
	/* Each line is defined separately with lb_line(..):             */
	lb_line(lb_ut3_ptr, 0, BLANK, "", 0);
	lb_line(lb_ut3_ptr, 1, RADIO, "Noise Source", &ra1);
	lb_line(lb_ut3_ptr, 2, RADIO, "Signal Source", &ra1);

	/* An edit field for Gain in Hz/dB */
	sprintf(eline1,"%d", near_int(v2f_gain)); /* The field to be edited */
	sl=strlen(eline1);
	lb_create_el(&el1, 7, 135, &eline1[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 3, EDIT, "Gain, Hz/dB", &el1);

	/* An edit field for Averaging Time: */
	sprintf(eline2,"%6.3lf", v2f_tc); /* The field to be edited */
	sl=strlen(eline2);
	lb_create_el(&el2, 7, 135, &eline2[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 5, EDIT, "Ave time, sec", &el2);

	/* An edit field for Base Tone Frequency: */
	sprintf(eline3,"%d", near_int(v2f_basef)); /* The field to be edited */
	sl=strlen(eline3);
	lb_create_el(&el3, 5, 151, &eline3[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 7, EDIT, "Base Tone, Hz", &el3);

	/* An edit field for Tone AF Level: */
	sprintf(eline6,"%d", v2f_af_level); /* The field to be edited */
	sl=strlen(eline6);
	lb_create_el(&el6, 3, 167, &eline6[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 9, EDIT, "Tone AF Lev 0-99", &el6);

	lb_line(lb_ut3_ptr, 11, TXT, " -- For Signals Only ---", 0);

	/* An edit field for Freq, low: */
	sprintf(eline4,"%d", near_int(v2f_freq_low)); /* Field to be edited */
	sl=strlen(eline4);
	lb_create_el(&el4, 5, 151, &eline4[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 12, EDIT, "Low Freq, Hz", &el4);

	/* An edit field for Freq, High: */
	sprintf(eline5, "%d", near_int(v2f_freq_high)); /* Field to be edited*/
	sl=strlen(eline5);
	lb_create_el(&el5, 5, 151, &eline5[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut3_ptr, 14, EDIT, "High Freq, Hz", &el5);

	if (v2f_pkave) bpush=17;   else bpush=16;
	lb_create_rb(&ra2, 0, bpush, 20);	/* Note: bpush pushed */
	/* Each line is defined separately with lb_line(..):             */
	lb_line(lb_ut3_ptr, 16, RADIO, "Ave Signal Pwr", &ra2);
	lb_line(lb_ut3_ptr, 17, RADIO, "Peak Signal Pwr", &ra2);
	lb_line(lb_ut3_ptr, 18, BLANK, "", 0);
	lb_line(lb_ut3_ptr, 19, BLANK, "", 0);
	lb_line(lb_ut3_ptr, 20, BLANK, "", 0);

	lb_update(lb_ut3_ptr);

        /* ----------- Loop ---------------- */
	/* Following is a loop for handling user keyboard input */

 r1:	rbui=lb_radio_button(lb_ut3_ptr, 1, &ra1);
	if     (rbui==CUR_UP)  goto r17;
	else if(rbui==ENTER)   goto lb_ut3end;

 r2:	rbui=lb_radio_button(lb_ut3_ptr, 2, &ra1);
	if     (rbui==CUR_UP)  goto r1;
	else if(rbui==ENTER)   goto lb_ut3end;

 e3:	edresult=lb_edit_line(lb_ut3_ptr, &el1);
	if (edresult==CUR_UP)   goto r2;
	else if (edresult==ENTER) goto lb_ut3end;

 e5:	edresult=lb_edit_line(lb_ut3_ptr, &el2);
	if (edresult==CUR_UP)   goto e3;
	else if (edresult==ENTER) goto lb_ut3end;

 e7:	edresult=lb_edit_line(lb_ut3_ptr, &el3);
	if (edresult==CUR_UP)   goto e5;
	else if (edresult==ENTER) goto lb_ut3end;

 e9:	edresult=lb_edit_line(lb_ut3_ptr, &el6);
	if (edresult==CUR_UP)   goto e7;
	else if (edresult==ENTER) goto lb_ut3end;

 e12:	edresult=lb_edit_line(lb_ut3_ptr, &el4);
	if (edresult==CUR_UP)   goto e9;
	else if (edresult==ENTER) goto lb_ut3end;

 e14:	edresult=lb_edit_line(lb_ut3_ptr, &el5);
	if (edresult==CUR_UP)   goto e12;
	else if (edresult==ENTER) goto lb_ut3end;

 r16:	rbui=lb_radio_button(lb_ut3_ptr, 16, &ra2);
	if     (rbui==CUR_UP)  goto e14;
	else if(rbui==ENTER)   goto lb_ut3end;

 r17:	rbui=lb_radio_button(lb_ut3_ptr, 17, &ra2);
	if     (rbui==CUR_UP)  goto r16;
	else if(rbui==ENTER)   goto lb_ut3end;

	goto r1;

 lb_ut3end:
	do_delay=FALSE;
	delay(200);
	/* As we exit, the variables must be changed to agree with the
	 * indicated states of the buttons and edit fields:
	 */

	if ((1&ra1.state[1])==1) v2f_noise=1;	/* Data source is WB noise */
	else                     v2f_noise=0;	/*  or signals from FFT bins */

	oldf=v2f_gain;
	v2f_gain=atof(eline1);
	if (v2f_gain<15.015)   { v2f_gain=15.015; do_delay=TRUE; }
	if (v2f_gain>3828.825) { v2f_gain=3828.825; do_delay=TRUE; }

	oldf=v2f_tc;
	v2f_tc=atof(eline2);
	if (v2f_noise && v2f_tc<0.01)  { v2f_tc=0.01; do_delay=TRUE; }
	if (!v2f_noise && v2f_tc<0.054){ v2f_tc=0.054; do_delay=TRUE; }
	if (v2f_tc>100.0) { v2f_tc=100.0; do_delay=TRUE; }

	oldf=v2f_basef;
	v2f_basef=atof(eline3);
	if (v2f_basef<100.0) { v2f_basef=100.0; do_delay=TRUE; }
	if (v2f_basef>999.0) { v2f_basef=999.0; do_delay=TRUE; }

	oldf=v2f_af_level;
	v2f_af_level=atoi(eline6);
	if (v2f_af_level<0)  { v2f_af_level=0;  do_delay=TRUE; }

	bin=df[fft_bw];
	half_bin=0.5*df[fft_bw];

	/* v2f_freq_low and _high are doubles that are the desired band limits.
	 * Achievable band limits must hit halfway between bin centers.
	 * So, find the bin and then the corresponding halfway point.     */
	oldf=v2f_freq_low;
	oldf2=v2f_freq_high;
	v2f_freq_low=atof(eline4);
	if (v2f_freq_low<half_bin)  v2f_freq_low=half_bin;  /* Exclude bin 0 */
	if (v2f_freq_low>510.0*bin) v2f_freq_low=510.0*bin;

	v2f_freq_high=atof(eline5);
	if (v2f_freq_high < bin)  v2f_freq_high=bin;
	if (v2f_freq_high > 511.0*bin) v2f_freq_high=511.0*bin;

	/* The band must be at least one bin wide. */
	vdf=v2f_freq_high-v2f_freq_low;	/* The requested width */
	if (vdf < bin)			/* Is it 1 bin minimum? */
	    {
	    v2f_freq_low -= (half_bin-0.5*vdf);
	    v2f_freq_high += (half_bin-0.5*vdf);
	    }

	/* Now place the frequency limits halfway between bins.
	 * v2f_bin_low and v2f_bin_high are integer bin numbers.
	 * First find the edge bins:  */
	v2f_bin_low=near_int((v2f_freq_low+half_bin-0.0001)/df[fft_bw]);
	v2f_bin_high=near_int((v2f_freq_high-half_bin+0.0001)/df[fft_bw]);
	/* 6 dB band limits are roughly halfway between bins (use windowing?)*/
	v2f_freq_low=bin*v2f_bin_low-half_bin;
	v2f_freq_high=bin*v2f_bin_high+half_bin;
	if(oldf != v2f_freq_low)  do_delay=TRUE;
	if(oldf2 != v2f_freq_high)  do_delay=TRUE;

	/* Final detail that the high bin measured is 1 less than that
	 * requested. If you want to measure bins 251 and 252, set _low=251
	 * and _high=253.     */
	v2f_bin_high++;

	if ((1&ra2.state[16])==1) v2f_pkave=0;	/* Signal bins are averaged */
	else                      v2f_pkave=1;	/*  or bin peak is found */

	if (do_delay)
	    {
	    lb_line(lb_ut3_ptr, 0, BOLD, " ******** WAIT *********", 0);
	    lb_line(lb_ut3_ptr, 19, BOLD, " ******** WAIT *********", 0);
	    }

	sprintf(eline1,"%d", near_int(v2f_gain));
	sprintf(eline2,"%6.3lf", v2f_tc);
	sprintf(eline3,"%d", near_int(v2f_basef));
	sprintf(eline6,"%d", v2f_af_level);
	sprintf(eline4,"%d", near_int(v2f_freq_low));
	sprintf(eline5, "%d", near_int(v2f_freq_high));
	lb_update(lb_ut3_ptr);		/* Re-display modified parameters */

	v2f(2);				/* Update all parameters */
	if (do_delay) delay(3000);	/* Show new values */
	delay(200);
	lb_close(lb_ut3_ptr, 1);
	free(lb_ut3_ptr);
	}

/*************************************************************************
 * A modal dialog box for Transmit Tone parameters.
 * The box structure is lb_ut7.
 */
void ttn_modalbox(void)
	{
	static struct check_info  ck1, ck2;
	static struct edit_info   el1, el2, el3, el4, el5, el6, el7;
	static struct radio_info  ra1;
	char eline1[12], eline2[12], eline3[12], eline4[12],
	    eline5[12], eline6[12], eline7[12];
	unsigned int ckui, edresult, rbui, bpush;
	int sl;
	struct lbox_info  *lb_ut7_ptr;
	int do_delay;
	double v1;			/* vi are temp variables */
	int v2, v3;

	/* Dynamic allocation of memory for box data */
	if ((lb_ut7_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
	lb_create(lb_ut7_ptr, "Transmit Tones",
			 160, 175, 370, 440, 0, MODAL);
	lb_open(lb_ut7_ptr, 1);

	lb_line(lb_ut7_ptr, 0, BLANK, "", 0);

	/* Lines 1 and 4 are the 2 members of the ra1 button family.
	 * Line 1 is DTMF, Line 4 is tone (CTCSS or Beep).
	 * lb_create_rb() sets the positions of the buttons and text, and
	 * sets the line with the pushed button. Only 1, of course.
	 */
	if(vfo[cur_vfo].t_fm_use == FMTONE_DTMF)
	    lb_create_rb(&ra1, 0, 1, 20);	/* Button 1 pushed */
	else
	    lb_create_rb(&ra1, 0, 4, 20);	/* Button 4 pushed */
	lb_line(lb_ut7_ptr, 1, RADIO, "DTMF from Keyboard", &ra1);
	lb_line(lb_ut7_ptr, 2, TXT, "  DTMF Level set below", 0);
	lb_line(lb_ut7_ptr, 3, BLANK, "", 0);

	lb_line(lb_ut7_ptr, 4, RADIO, "FM Tone", &ra1);

	/* An edit field for FM Tone Freq */
	sprintf(eline2,"%.1lf", vfo[cur_vfo].t_fm_freq); /* The edit field */
	sl=strlen(eline2);
	lb_create_el(&el2, 7, 135, &eline2[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 6, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 5, EDIT, "  Tone Freq, Hz", &el2);

	/* An edit field for FM Tone Duration */
	sprintf(eline3,"%d", vfo[cur_vfo].t_fm_dur);
	sl=strlen(eline3);
	lb_create_el(&el3, 5, 151, &eline3[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 8, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 7, EDIT, "  Duration, ms", &el3);
	lb_line(lb_ut7_ptr, 9, TXT,  "  For CTCSS Duration=9999", 0);

	/* An edit field for FM Tone Level */
	sprintf(eline4,"%d", vfo[cur_vfo].t_fm_level);
	sl=strlen(eline4);
	lb_create_el(&el4, 4, 159, &eline4[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 11, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 10, EDIT, "  Level (0,100)", &el4);

	lb_line(lb_ut7_ptr, 12, BOLD, " -----------------------", 0);

	if (beep_use & 1)		/* Show box checked */
	    lb_create_check(&ck1, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck1, 0, FALSE, 20);
	lb_line(lb_ut7_ptr, 13, CHECK, "SSB End Beep", &ck1);

	/* An edit field for Beep Tone Freq */
	sprintf(eline5,"%.1lf", beep_freq); /* The field to be edited */
	sl=strlen(eline5);
	lb_create_el(&el5, 7, 135, &eline5[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 15, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 14, EDIT, "  End-Beep Freq", &el5);

	/* An edit field for Beep Tone Duration */
	sprintf(eline6,"%d", beep_dur); /* The field to be edited */
	sl=strlen(eline6);
	lb_create_el(&el6, 5, 151, &eline6[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 17, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 16, EDIT, "  Duration, ms", &el6);

	/* An edit field for Beep Tone Level */
	sprintf(eline7,"%d", beep_level); /* The field to be edited */
	sl=strlen(eline7);
	lb_create_el(&el7, 4, 159, &eline7[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 19, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 18, EDIT, "  Level (0,100)", &el7);

	/* Check box to see if a side tone should accompany end beep */
	if (beep_use & 2)		/* Show box checked */
	    lb_create_check(&ck2, 12, TRUE, 32);
	else				/* Show box unchecked */
	    lb_create_check(&ck2, 12, FALSE, 32);
	lb_line(lb_ut7_ptr, 20, CHECK, "End Beep Sidetone", &ck2);

	/* Fixed text in modal box:     */
	lb_line(lb_ut7_ptr, 21, BOLD, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut7_ptr, 22, BOLD, "SPACE changes checks", 0);
	lb_line(lb_ut7_ptr, 23, BOLD, "\030 and \031 Move to Next Item", 0);
	lb_line(lb_ut7_ptr, 24, BOLD, "ENTER=Close    ESC=Cancel", 0);

        /* ----------- Loop ---------------- */
	/* Following is a loop for handling user keyboard input */

	/* lb_radio_button makes the designated line active (bright line)
	 * and does lb_update(). It then waits for an action keyboard input,
	 * as space to change state, Enter to exit box, up or down
	 * cursor.
	 */
 r1:	rbui=lb_radio_button(lb_ut7_ptr, 1, &ra1);
	if     (rbui==CUR_UP)  goto c20;
	else if(rbui==ENTER)   goto lb_ut7end;

#if 0
 e2:	edresult=lb_edit_line(lb_ut7_ptr, &el1);
	if (edresult==CUR_UP)   goto r1;
	else if (edresult==ENTER) goto lb_ut7end;
#endif

 r4:	rbui=lb_radio_button(lb_ut7_ptr, 4, &ra1);
	if     (rbui==CUR_UP)  goto r1;
	else if(rbui==ENTER)   goto lb_ut7end;

 e5:	edresult=lb_edit_line(lb_ut7_ptr, &el2);
	if (edresult==CUR_UP)   goto r4;
	else if (edresult==ENTER) goto lb_ut7end;

 e7:	edresult=lb_edit_line(lb_ut7_ptr, &el3);
	if (edresult==CUR_UP)   goto e5;
	else if (edresult==ENTER) goto lb_ut7end;

 e10:	edresult=lb_edit_line(lb_ut7_ptr, &el4);
	if (edresult==CUR_UP)   goto e7;
	else if (edresult==ENTER) goto lb_ut7end;

 c13:	ckui=lb_check_mark(lb_ut7_ptr, &ck1);
	if (ckui==CUR_UP)  goto e10;
	else if (ckui==ENTER)  goto lb_ut7end;

 e14:	edresult=lb_edit_line(lb_ut7_ptr, &el5);
	if (edresult==CUR_UP)   goto c13;
	else if (edresult==ENTER) goto lb_ut7end;

 e16:	edresult=lb_edit_line(lb_ut7_ptr, &el6);
	if (edresult==CUR_UP)   goto e14;
	else if (edresult==ENTER) goto lb_ut7end;

 e18:	edresult=lb_edit_line(lb_ut7_ptr, &el7);
	if (edresult==CUR_UP)   goto e16;
	else if (edresult==ENTER) goto lb_ut7end;

 c20:	ckui=lb_check_mark(lb_ut7_ptr, &ck2);
	if (ckui==CUR_UP)  goto e18;
	else if (ckui==ENTER)  goto lb_ut7end;

	goto r1;

 lb_ut7end:
	do_delay=FALSE;
	delay(200);
	/* As we exit, the variables must be changed to agree with the
	 * indicated states of the buttons and edit fields:
	 */
	if ((1 & ra1.state[1]) == 1)
	    vfo[cur_vfo].t_fm_use=FMTONE_DTMF;	/* DTMF, not CTCSS or Beep */
	else
	    vfo[cur_vfo].t_fm_use=FMTONE_TONE;	/* CTCSS or Beep, not DTMF */

	v1=atof(eline2);
	if (v1<0.0)  { v1=0.0; do_delay=TRUE; }
	if (v1>3500.0)  { v1=3500.0; do_delay=TRUE; }
	vfo[cur_vfo].t_fm_freq=v1;

	v2=atoi(eline3);
	if (v2<0)  { v2=0; do_delay=TRUE; }
	if (v2>9999)  { v2=9999; do_delay=TRUE; }
	vfo[cur_vfo].t_fm_dur=v2;

	v3=atoi(eline4);
	if (v3<0)  { v3=0; do_delay=TRUE; }
	if (v3>100)  { v3=100; do_delay=TRUE; }
	vfo[cur_vfo].t_fm_level=v3;

	if (ck1.state & CHECK_MARK)
	    beep_use |= 1;		/* Set bit 0 */
	else
	    beep_use &= 0XFFFE;		/* Clear bit 0 */

	beep_freq=atof(eline5);
	if (beep_freq<0.0)  { beep_freq=0.0; do_delay=TRUE; }
	if (beep_freq>3500.0)  { beep_freq=3500.0; do_delay=TRUE; }

	beep_dur=atoi(eline6);
	if (beep_dur<0)  { beep_dur=0; do_delay=TRUE; }
	if (beep_dur>9999)  { beep_dur=9999; do_delay=TRUE; }

	beep_level=atoi(eline7);
	if (beep_level<0)  { beep_level=0; do_delay=TRUE; }
	if (beep_level>100)  { beep_level=100; do_delay=TRUE; }

	if (ck2.state & CHECK_MARK)
	    beep_use |= 2;		/* Set bit 1 */
	else
	    beep_use &= 0XFFFD;		/* Clear bit 1 */

	if (do_delay)			/* Display the changes */
	    {
	    beep(500);			/* Because a change has been made */
	    lb_line(lb_ut7_ptr, 0, BOLD, " ******** WAIT *********", 0);
	    sprintf(eline2,"%.1lf", vfo[cur_vfo].t_fm_freq);
	    sprintf(eline3,"%d", vfo[cur_vfo].t_fm_dur);
	    sprintf(eline4,"%d", vfo[cur_vfo].t_fm_level);
	    sprintf(eline5,"%.1lf", beep_freq);
	    sprintf(eline6,"%d", beep_dur);
	    sprintf(eline7,"%d", beep_level);
	    lb_update(lb_ut7_ptr);	/* Re-display modified parameters */
	    delay(3000);
	    }
	delay(200);
	lb_close(lb_ut7_ptr, 1);
	free(lb_ut7_ptr);
	}				/* End ttn_modalbox()    */

void beep(int tau)
	{sound(600);
	delay(tau);
	nosound();
	}

void quit(void)
	{
	if (transmit)  t_r_toggle();  /* Leave hardware in rcv */
	if (mode==FM) do_window=do_window_save;  /* Save SSB window */
	/* Now program DSP for fft_fcn as 1024 pt, 4800 Hz, Free run */
	dsp_cmd('F', 0X44, 0X03, 0, 4);	/* Triggered, clr: 'F', hi, lo, 0, 4*/
	delay(100);
	dsp_cmd('F', 0X44, 0X00, 0, 4);	/* Un-Triggered */

	farfree( big_area_ptr1 );
	farfree( big_area_ptr2 );
	free( power10 );
	free( pwrdat );
	free( dnorm );
	free( lt_dnorm );
	free( lt_pwrdat );
	free( pd16 );
	pua43_update(0, -2);		/* De allocate memory */

	dsp_cmd('J', 1, 0, 1, 4);	/* Byte 3=sys_ctrl; leave 1 for "$$$"*/
	ASYNCSTOP();
	setallpalette(&savpalette);	/* The palette when we got here */
	setbkcolor(BLACK);
	setcolor(WHITE);
	closegraph();

	if ( quit_save_state && !floppy_door(deflt_drv) )
	    {
	    beep(500); delay(500); beep(500);
	    printf("\nWARNING: Unable to write to Floppy Drive to save Transceiver State.");
	    printf("\nAfter correcting problem, hit any key. File will be written if possible.");
	    getch();
	    }
	if ( quit_save_state && floppy_door(deflt_drv) )
	    {
	    printf("\nSaving Transceiver state to UHFA.CFG configuration file");
	    save_state();
	    }
	printf("\nExiting DSP-10 uhfa.exe program\n");
	exit(0);
	}
