/* u_code.h  Header for the routines used to send and receive various
 * codes.
 * Initial write     RSL  6 Dec 98
 * Added struct lhl + declare lhl and pua functions.  RSL 21 Jan 99
 */

/* The 7 frequencies for the LHL-7 Code;  */
#define		LHL_SEP		600
#define		LHL_DOT		750
#define		LHL_DASH	900
#define		LHL_X2		1050
#define		LHL_X3		1200
#define		LHL_X4		1350
#define		LHL_X5		1500

#define		LHL_CWID	500	/* For CW identification */

extern void morse_ch(unsigned char,		/* Character to be sent */
	      int);			/* Code speed index (0, 25) */
extern void morse(int, int);		/* Code routines, (command, ichar) */
extern void put_morse(int);		/* Put message into CW buffer */
extern int  lhl_code(unsigned char, struct lhl*); /* Transmit LHL Code */
extern double pua43_freq(unsigned char);	/* Transmit PUA43 Code */
extern double lti_freq(void);			/* Randomize LTI */
extern void lhl7_update(int, int);		/* Receive LHL Code */
extern void set_code_min(void);			/* Sets global code_min */
extern void pua43_update(int, int);		/* Receive PUA43 Code */
extern void eme2_update(int);			/* Receive eme 2 sec transmissions */
extern void lterm_int_update(int);		/* Long term integration in USB */
extern void eme2_modalbox(void);
extern void lhl7_modalbox(void);
extern void pua43_modalbox(void);
extern void pua43_delete_modalbox(void);
extern void lterm_int_modalbox(void);
extern void sigsim_modalbox(void);

/* Note:  Code speed can be any.  There are 4 times in milliseconds that
 * set the actual speed and weighting.  The index (0, 25) allows 26 of these
 * that are initialized to 5 WPM to 30 WPM with nominal weightings.
 *
 * The following set the weightings vs. code speed in wpm
 * by selecting the delays for
 * .dot   length of a dot in ms
 * .dash  length of a dash in ms
 * .dd    time between dots and dashes, ms
 * .ch    time between characters of the same word, ms
 * .sp    time duration between words, ms
 */
struct cw_data {
	char	description[7];		/* String for panel display */
	int	dot;			/* msec per dot */
	int	dash;			/* msec per dash */
	int	dd;			/* msec between dot or dash */
	int	ch;			/* msec at end of each char */
	int	sp;			/* msec for space char */
	};

extern struct cw_data t[];

/* The LHL code is built around the idea of assigning a tone to the number
 * of repititions of a dot or dash occurs in Morse code. This ends up with
 * seven tones, with the possibility that one of the tones could be a
 * key up condition. As this is being sent, the tones for a character, as
 * well as the current state are carried in a structure:
 */
struct lhl {
	unsigned char cc;               /* Eight bit  char */
	int      ntone;                 /* Number tones to represent ch */
	int      curtone;               /* Next tone, 0 at start */
	int      tone[11];              /* tone frequencies to be sent */
	};

extern struct lbox_info   lb_eme2;	  /* Info box, upper-right of w'fall*/
extern struct lbox_info   lb_lhl7;
extern struct lbox_info   lb_pua43;
extern struct lbox_info   lb_lterm_int;
